package com.sophiaatkinson.pronouns;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.bukkit.plugin.java.JavaPlugin;

public class PronounsPlugin extends JavaPlugin {
    private Connection connection;
    private List<String> blockedPronouns;
    private List<String> pronounSuggestions;
    private final List<String> errorLog = new ArrayList<>();

    @Override
    public void onEnable() {
        getLogger().info("PronounsPlugin enabled!");

        // Load blocked pronouns and pronoun suggestions from config
        saveDefaultConfig();
        blockedPronouns = getConfig().getStringList("blocked-pronouns");
        pronounSuggestions = getConfig().getStringList("pronoun-suggestions");

        // Load database
        initDatabase();

        // Register commands and placeholders
        PronounsCommand command = new PronounsCommand(this);
        getCommand("pronouns").setExecutor(command);
        getCommand("pronouns").setTabCompleter(command);

        new PronounsPlaceholder(this).register();
    }

    @Override
    public void onDisable() {
        try {
            if (connection != null) connection.close();
        } catch (SQLException e) {
            errorLog.add("Database close error: " + e.getMessage());
            e.printStackTrace();
        }
    }

    private void initDatabase() {
        try {
            File dbFile = new File(getDataFolder(), "pronouns.db");
            if (!getDataFolder().exists()) getDataFolder().mkdirs();

            String url = "jdbc:sqlite:" + dbFile.getAbsolutePath();
            connection = DriverManager.getConnection(url);

            Statement stmt = connection.createStatement();
            stmt.executeUpdate("CREATE TABLE IF NOT EXISTS pronouns (uuid TEXT PRIMARY KEY, pronouns TEXT);");
        } catch (SQLException e) {
            errorLog.add("Database initialization error: " + e.getMessage());
            e.printStackTrace();
        }
    }

    public void setPronouns(UUID uuid, String pronouns) {
        if (pronouns == null) {
            try (PreparedStatement ps = connection.prepareStatement("DELETE FROM pronouns WHERE uuid = ?")) {
                ps.setString(1, uuid.toString());
                ps.executeUpdate();
            } catch (SQLException e) {
                errorLog.add("Database delete error: " + e.getMessage());
                e.printStackTrace();
            }
            return;
        }
    
        for (String blocked : blockedPronouns) {
            if (pronouns.contains(blocked)) {
                return;
            }
        }

        try (PreparedStatement ps = connection.prepareStatement("REPLACE INTO pronouns (uuid, pronouns) VALUES (?, ?)")) {
            ps.setString(1, uuid.toString());
            ps.setString(2, pronouns);
            ps.executeUpdate();
        } catch (SQLException e) {
            errorLog.add("Database update error: " + e.getMessage());
            e.printStackTrace();
        }
    }

    public String getPronouns(UUID uuid) {
        try (PreparedStatement ps = connection.prepareStatement("SELECT pronouns FROM pronouns WHERE uuid = ?")) {
            ps.setString(1, uuid.toString());
            ResultSet rs = ps.executeQuery();
            if (rs.next()) {
                return rs.getString("pronouns");
            }
        } catch (SQLException e) {
            errorLog.add("Database fetch error: " + e.getMessage());
            e.printStackTrace();
        }
        return null;
    }

    public void generateDebugFile() {
        File debugFile = new File(getDataFolder(), "debug.txt");
        try {
            if (!getDataFolder().exists()) getDataFolder().mkdirs();
            if (!debugFile.exists()) debugFile.createNewFile();

            FileWriter writer = new FileWriter(debugFile);
            writer.write("=== PronounsPlugin Debug Report ===\n");
            writer.write("Plugin Enabled: " + isEnabled() + "\n");
            writer.write("Database Connected: " + (connection != null) + "\n");
            writer.write("Blocked Pronouns: " + blockedPronouns + "\n");
            writer.write("Pronoun Suggestions: " + pronounSuggestions + "\n");
            writer.write("Error Log:\n");

            if (errorLog.isEmpty()) {
                writer.write("  No errors logged.\n");
            } else {
                for (String error : errorLog) {
                    writer.write("  - " + error + "\n");
                }
            }

            writer.write("Generated at: " + new java.util.Date() + "\n");
            writer.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public List<String> getPronounSuggestions() {
        return pronounSuggestions;
    }
}
